#ifndef SCENE_H
#define SCENE_H

#include <list>

#include "Model.h"
#include "Camera.h"
#include "Animation.h"

#include <GLM/gtc/type_ptr.hpp>
#include <GLM/gtc/quaternion.hpp>
#include <GLM/gtx/quaternion.hpp>
#include <GLM/gtx/euler_angles.hpp>

#define DEFAULT_MTL_DIR "./"
class Scene loadOBJtoScene(const char* filename, const char* mtl_directory = DEFAULT_MTL_DIR, const char* texture_directory = DEFAULT_MTL_DIR);
class Scene loadGLTFtoScene(std::string filename);

// Класс сцены
class Scene
{
    public:
        Scene(); // Конструктор пустой сцены
        Scene(const Scene &copy); // Конструктор копирования
        Scene& operator=(const Scene& other); // Оператор присваивания

        void render(ShaderProgram &shaderProgram, UBO &material_buffer, bool recalc_animations = false); // Рендер сцены

        void set_group_id(GLuint64 id, GLuint etc = 0); // Изменение флага записи идентификатора для всех моделей

        Node root; // Корневой узел

        // Списки объектов, выступающих узлами
        std::list<Node> nodes; // Список пустых узлов
        std::list<Model> models; // Список моделей для рендера
        std::list<Camera> cameras; // Список камер

        std::vector<Animation> animations; // Список анимаций
        std::map<std::string, size_t> animation_names; // Имя анимации - индекс
    protected:
        void rebuld_tree(const Scene& from); // Перестройка дерева после копирования или присваивания
        template <class T>
        void rebuild_Nodes_list(T& nodes, const Scene& from); // Перестройка узлов выбранного списка
        template <class T>
        void move_parent(Node& for_node, const std::list<T>& from_nodes, std::list<T>& this_nodes); // Сдвигает родителя узла между двумя списками при условии его принадлежности к оригинальному
        template <class T>
        void move_animation_target(Node*& target, const std::list<T>& from_nodes, std::list<T>& this_nodes); // Перестройка узлов анимации
};

#endif // SCENE_H
